@@ -19,6 +19,8 @@ gint64 key_press_time, key_press_time_ctrl;
 extern gboolean b_hsu_kbm;
 extern gboolean test_mode;
 extern char *pho_chars[];
+extern char *wselkey;
+void set_wselkey();
 
 TSIN_ST tss;
 
@@ -34,24 +36,28 @@ extern int hashidx[TSIN_HASH_N];
 void clrin_pho(), hide_win0();
 void show_tsin_stat();
 void save_CS_current_to_temp();
+gboolean typ_pho_empty3();
 
 gboolean tsin_pho_mode()
 {
   return current_CS && current_CS->tsin_pho_mode;
 }
 
-void set_tsin_pho_mode0(ClientState *cs)
+void disp_tray_icon();
+
+void set_tsin_pho_mode0(ClientState *cs, gboolean tsin_pho_mode)
 {
   if (!cs)
     return;
-  cs->tsin_pho_mode = 1;
+  cs->tsin_pho_mode = tsin_pho_mode;
   save_CS_current_to_temp();
 }
 
 void set_tsin_pho_mode()
 {
-  set_tsin_pho_mode0(current_CS);
+  set_tsin_pho_mode0(current_CS, TRUE);
   show_tsin_stat();
+  set_wselkey();
 }
 
 gboolean tsin_cursor_end()
@@ -177,9 +183,12 @@ void chpho_get_str(int idx, int len, char *ch)
 
 void inc_pho_count(phokey_t key, int ch_idx);
 int ch_key_to_ch_pho_idx(phokey_t phkey, char *big5);
-void inc_dec_tsin_use_count(void *pho, char *ch, int prlen);
+gboolean inc_tsin_use_count(TSIN_HANDLE *th, void *pho, char *ch, int prlen);
+gboolean inc_tsin_use_count_en(char *s, int len);
 void lookup_gtabn(char *ch, char *);
 
+
+
 static void putbuf(int len)
 {
   u_char tt[CH_SZ * (MAX_PH_BF_EXT+1) + 1];
@@ -211,9 +220,14 @@ static void putbuf(int len)
       phokey_t pho[MAX_PHRASE_LEN];
       char ch[MAX_PHRASE_LEN * CH_SZ * 2];
 
-      chpho_extract(&tss.chpho[i], phrlen, pho, ch);
-
-      inc_dec_tsin_use_count(pho, ch, phrlen);
+	  if (BITON(tss.chpho[i].flag, FLAG_CHPHO_EN_PHRASE)) {
+		dbg("FLAG_CHPHO_EN_PHRASE\n");
+        extract_pho(TRUE, i, phrlen, pho);
+		inc_tsin_use_count_en((char *)pho, phrlen);
+	  } else {
+        chpho_extract(&tss.chpho[i], phrlen, pho, ch);
+        inc_tsin_use_count(&tsin_hand, pho, ch, phrlen);
+      }
     }
   }
 #endif
@@ -364,6 +378,7 @@ void tsin_reset_in_pho()
 
 void flush_tsin_buffer()
 {
+//  dbg("flush_tsin_buffer\n");
 #if WIN32
   if (test_mode)
     return;
@@ -487,7 +502,25 @@ static void move_cursor_end()
   drawcursor();
 }
 
-gboolean save_phrase_to_db2(CHPHO *chph, int len);
+gboolean save_phrase_to_db2(CHPHO *chph, int len, gboolean en_prhase);
+phokey_t utf8_pho_key(char *s)
+{
+  phokey_t tphokeys[32];
+  tphokeys[0]=0;
+  utf8_pho_keys(s, tphokeys);
+  return tphokeys[0];
+}
+
+gboolean is_legal_en_char(char *ch)
+{
+  char c = ch[0];
+#if 0
+  return c>='A' && c<='Z' || c>='a' && c<='z' || strchr(" '-_+.,0123456789@()&$!#", c) != NULL;
+#else
+  return !(c&0x80);
+#endif
+}
+
 
 void save_phrase(int save_frm, int len)
 {
@@ -500,22 +533,23 @@ void save_phrase(int save_frm, int len)
   if (len <= 0 || len > MAX_PHRASE_LEN)
     return;
 
+  dbg("save_phrase\n");
+
+  gboolean en_phrase = TRUE;
   int i;
   for(i=save_frm;i<=save_to;i++) {
+    if (!is_legal_en_char(tss.chpho[i].ch))
+       en_phrase = FALSE;
+
     if (tss.chpho[i].pho)
       continue;
-    phokey_t tpho[32];
-    tpho[0]=0;
-
-    utf8_pho_keys(tss.chpho[i].ch, tpho);
-
-    if (!tpho[0])
+    phokey_t tk = utf8_pho_key(tss.chpho[i].ch);
+    if (!tk)
       return;
-
-    tss.chpho[i].pho = tpho[0];
+    tss.chpho[i].pho = tk;
   }
 
-  if (!save_phrase_to_db2(&tss.chpho[save_frm], len)) {
+  if (!save_phrase_to_db2(&tss.chpho[save_frm], len, en_phrase)) {
     bell();
   }
 
@@ -638,8 +672,6 @@ static void put_u8_char(int pho_idx, phokey_t key, gboolean b_tone)
 static u_char selstr[MAX_PHRASE_SEL_N][MAX_PHRASE_LEN * CH_SZ];
 static u_char sellen[MAX_PHRASE_SEL_N];
 
-static u_short phrase_count;
-static u_short pho_count;
 
 static gboolean chpho_eq_pho(int idx, phokey_t *phos, int len)
 {
@@ -684,7 +716,7 @@ static void get_sel_phrase0(int selidx, gboolean eqlen)
     mlen=MAX_PHRASE_LEN;
 
   phokey_t pp[MAX_PHRASE_LEN + 1];
-  extract_pho(selidx, mlen, pp);
+  extract_pho(FALSE, selidx, mlen, pp);
 
   char *pinyin_s = NULL;
   char pinyin_set[MAX_PH_BF_EXT];
@@ -695,7 +727,7 @@ static void get_sel_phrase0(int selidx, gboolean eqlen)
   if (!tsin_seek(pp, 2, &sti, &edi, pinyin_s))
     return;
 
-  while (sti < edi && phrase_count < phkbm.selkeyN) {
+  while (sti < edi && tss.phrase_count < phkbm.selkeyN) {
     phokey_t stk[MAX_PHRASE_LEN];
     usecount_t usecount;
     u_char stch[MAX_PHRASE_LEN * CH_SZ + 1];
@@ -709,8 +741,8 @@ static void get_sel_phrase0(int selidx, gboolean eqlen)
     }
 
     if (chpho_eq_pho(selidx, stk, len)) {
-      sellen[phrase_count]=len;
-      utf8cpyN((char *)selstr[phrase_count++], (char *)stch, len);
+      sellen[tss.phrase_count]=len;
+      utf8cpyN((char *)selstr[tss.phrase_count++], (char *)stch, len);
     }
 
     sti++;
@@ -723,7 +755,7 @@ static void get_sel_phrase_end()
   if (stidx < 0)
     stidx = 0;
 
-  phrase_count = 0;
+  tss.phrase_count = 0;
   int i;
   for(i=stidx; i < tss.c_len - 1; i++) {
     get_sel_phrase0(i, TRUE);
@@ -732,7 +764,7 @@ static void get_sel_phrase_end()
 
 static void get_sel_phrase()
 {
-  phrase_count = 0;
+  tss.phrase_count = 0;
   get_sel_phrase0(tss.c_idx, FALSE);
 }
 
@@ -783,13 +815,13 @@ static void get_sel_pho()
   } else
     end = idx_pho[i+1].start;
 
-  pho_count = end - tss.startf;
-//  dbg("pho_count %d\n", pho_count);
+  tss.pho_count = end - tss.startf;
+//  dbg("tss.pho_count %d\n", tss.pho_count);
 }
 
 
 void clear_sele();
-void set_sele_text(int tN, int i, char *text, int len);
+void set_sele_text(int i, char *text, int len);
 void disp_arrow_up(), disp_arrow_down();
 void disp_tsin_select(int index);
 
@@ -802,19 +834,19 @@ static void disp_current_sel_page()
   for(i=0; i < phkbm.selkeyN; i++) {
     int idx = tss.current_page + i;
 
-    if (idx < phrase_count) {
+    if (idx < tss.phrase_count) {
       int tlen = utf8_tlen((char *)selstr[i], sellen[i]);
-      set_sele_text(phrase_count + pho_count, i, (char *)selstr[i], tlen);
+      set_sele_text(i, (char *)selstr[i], tlen);
     } else
-    if (idx < phrase_count + pho_count) {
-      int v = idx - phrase_count + tss.startf;
+    if (idx < tss.phrase_count + tss.pho_count) {
+      int v = idx - tss.phrase_count + tss.startf;
       char *tstr = pho_idx_str(v);
-      set_sele_text(phrase_count + pho_count, i, tstr, -1);
+      set_sele_text(i, tstr, -1);
     } else
       break;
   }
 
-  if (tss.current_page + phkbm.selkeyN < phrase_count + pho_count) {
+  if (tss.current_page + phkbm.selkeyN < tss.phrase_count + tss.pho_count) {
     disp_arrow_down();
   }
 
@@ -830,12 +862,12 @@ static int fetch_user_selection(int val, char **seltext, int *is_pho_phrase)
   int len = 0;
 
   *is_pho_phrase = FALSE;
-  if (idx < phrase_count) {
+  if (idx < tss.phrase_count) {
     len = sellen[idx];
     *seltext = (char *)selstr[idx];
   } else
-  if (idx < phrase_count + pho_count) {
-    int v = idx - phrase_count + tss.startf;
+  if (idx < tss.phrase_count + tss.pho_count) {
+    int v = idx - tss.phrase_count + tss.startf;
     *seltext = pho_idx_str2(v, is_pho_phrase);
     len = utf8_str_N(*seltext);
   }
@@ -844,12 +876,20 @@ static int fetch_user_selection(int val, char **seltext, int *is_pho_phrase)
 }
 
 
-void extract_pho(int chpho_idx, int plen, phokey_t *pho)
+void extract_pho(gboolean is_en, int chpho_idx, int plen, phokey_t *pho)
 {
   int i;
 
-  for(i=0; i < plen; i++) {
-    pho[i] = tss.chpho[chpho_idx + i].pho;
+  if (is_en) {
+	char *asc = (char *)pho;
+	int tlen = 0;
+    for(i=0; i < plen; i++) {
+      tlen+=u8cpy(asc+tlen,  tss.chpho[chpho_idx + i].ch);
+    }
+  }	else {
+    for(i=0; i < plen; i++) {
+      pho[i] = tss.chpho[chpho_idx + i].pho;
+    }
   }
 }
 
@@ -898,7 +938,7 @@ void disp_pre_sel_page()
   for(i=0; i < tss.pre_selN; i++) {
     int tlen = utf8_tlen(tss.pre_sel[i].str, tss.pre_sel[i].len);
 
-    set_sele_text(tss.pre_selN, i, tss.pre_sel[i].str, tlen);
+    set_sele_text(i, tss.pre_sel[i].str, tlen);
   }
 
 #if 0
@@ -940,13 +980,18 @@ void tsin_set_eng_ch(int nmod)
   }
 
   show_tsin_stat();
+  set_wselkey();
 }
 
+void load_en_db();
+
 void tsin_toggle_eng_ch()
 {
 //  dbg("tsin_toggle_eng_ch\n");
   compact_win0();
   tsin_set_eng_ch(!tsin_pho_mode());
+  //if (!en_hand.fph)
+	//load_en_db();
 }
 
 
@@ -960,12 +1005,13 @@ void tsin_toggle_half_full()
 #endif
 }
 
-
+#if 0
 #if 0
 static char ochars[]="<,>.?/:;\"'{[}]_-+=|\\~`";
 #else
 static char ochars[]="<,>.?/:;\"'{[}]_-+=|\\";
 #endif
+#endif
 
 void hide_pre_sel()
 {
@@ -997,6 +1043,8 @@ void ch_pho_cpy(CHPHO *pchpho, char *utf8, phokey_t *phos, int len)
   for(i=0; i < len; i++) {
     int len = utf8cpy(pchpho[i].cha, utf8);
     utf8+=len;
+    if (!phos[i])
+      phos[i] = utf8_pho_key(pchpho[i].cha);
     pchpho[i].pho = phos[i];
 	pchpho[i].flag &= ~FLAG_CHPHO_PHO_PHRASE;
   }
@@ -1016,6 +1064,10 @@ void set_chpho_ch(CHPHO *pchpho, char *utf8, int len, gboolean is_pho_phrase)
       u8len = utf8cpy(pchpho[i].cha, utf8);
       pchpho[i].ch = pchpho[i].cha;
 	  pchpho[i].flag &= ~FLAG_CHPHO_PHO_PHRASE;
+      if (!tsin_pho_mode()) {
+//		dbg("set en %d\n", i);
+        pchpho[i].flag |= FLAG_CHPHO_EN_PHRASE;
+      }
     }
 
     utf8+=u8len;
@@ -1087,14 +1139,27 @@ static void set_phrase_link(int idx, int len)
 }
 #endif
 
+void dump_flag()
+{
+  int i;
+  for(i=0;i<tss.c_len;i++) {
+	dbg("chpho %d] %x\n", i, tss.chpho[i].flag);
+  }
+}
 
 // should be used only if it is a real phrase
 gboolean add_to_tsin_buf_phsta(char *str, phokey_t *pho, int len)
 {
     int idx = tss.ph_sta;
-#if 0
-    dbg("idx:%d  tss.ph_sta:%d tss.ph_sta_last:%d tss.c_idx:%d  tss.c_len:%d\n",
-       idx, tss.ph_sta, tss.ph_sta_last, tss.c_idx, tss.c_len);
+
+    if (!tsin_pho_mode()) { // len of En preselect is strlen, not Uni char len
+	   len = utf8_str_N(str);
+	   dbg("len:%d\n", len);
+	}
+
+#if 1
+    dbg("add_to_tsin_buf_phsta idx:%d tss.c_idx:%d  tss.c_len:%d pho_mode:%d\n",
+       idx, tss.c_idx, tss.c_len, tsin_pho_mode());
 #endif
     if (idx < 0)
       return 0;
@@ -1116,6 +1181,7 @@ gboolean add_to_tsin_buf_phsta(char *str, phokey_t *pho, int len)
 
     ch_pho_cpy(&tss.chpho[idx], str, pho, len);
     set_chpho_ch(&tss.chpho[idx], str, len, FALSE);
+//    dump_flag();
     set_fixed(idx, len);
     tss.chpho[idx].flag |= FLAG_CHPHO_PHRASE_USER_HEAD;
     tss.c_idx=idx + len;
@@ -1132,7 +1198,9 @@ gboolean add_to_tsin_buf_phsta(char *str, phokey_t *pho, int len)
     disp_ph_sta();
     hide_pre_sel();
     tss.ph_sta=-1;
-    call_tsin_parse();
+
+	if (tsin_pho_mode())
+      call_tsin_parse();
 
     return 1;
 }
@@ -1144,6 +1212,7 @@ void add_to_tsin_buf_str(char *str)
   char *endp = pp+strlen(pp);
   int N = 0;
 
+  dbg("add_to_tsin_buf_str %s\n", str);
 
   while (*pp) {
     int u8sz = utf8_sz(pp);
@@ -1154,17 +1223,33 @@ void add_to_tsin_buf_str(char *str)
       break;
   }
 
-  dbg("add_to_tsin_buf_str %s %d\n",str, N);
+//  dbg("add_to_tsin_buf_str %s %d\n",str, N);
 
   phokey_t pho[MAX_PHRASE_LEN];
+#if 0
   bzero(pho, sizeof(pho));
+#else
+  int i;
+  for(i=0;i<N;i++)
+	  pho[i]=0x1; //˙
+#endif
   add_to_tsin_buf(str, pho, N);
 }
 
+void add_to_tsin_buf_ch(char c)
+{
+	dbg("add_to_tsin_buf_ch %c\n", c);
+	char tt[2];
+	tt[0]=c;
+	tt[1]=0;
+	add_to_tsin_buf_str(tt);
+}
+
 int tsin_pho_sel(int c);
 
 int tsin_sele_by_idx(int c)
 {
+  dbg("tsin_sele_by_idx %d\n", c);
   if (tss.sel_pho) {
     tsin_pho_sel(c);
     return 0;
@@ -1185,8 +1270,20 @@ int tsin_sele_by_idx(int c)
   return b_added;
 }
 
-static char shift_sele[]="!@#$%^&*()asdfghjkl:zxcvbnm<>?qwertyuiop";
-static char noshi_sele[]="1234567890asdfghjkl;zxcvbnm,./qwertyuiop";
+char shift_sele[]="!@#$%^&*()asdfghjkl:zxcvbnm<>?qwertyuiop{}";
+char noshi_sele[]="1234567890asdfghjkl;zxcvbnm,./qwertyuiop[]";
+
+char strip_shift_key(KeySym xkey)
+{
+  if (xkey >= 127)
+    return 0;
+  char *p;
+  if (!(p=strchr(shift_sele, xkey)))
+    return 0;
+  int c = p - shift_sele;
+  return noshi_sele[c];
+}
+
 int shift_key_idx(char *s, KeySym xkey)
 {
   if (xkey >= 0x7f)
@@ -1212,11 +1309,23 @@ int shift_key_idx(char *s, KeySym xkey)
 }
 
 
-static gboolean pre_sel_handler(KeySym xkey)
+static gboolean pre_sel_handler(KeySym xkey, int kbstate)
 {
+//  dbg("pre_sel_handler\n");
   if (!tss.pre_selN || !tsin_phrase_pre_select)
     return FALSE;
 
+  if (xkey > 127)
+    return FALSE;
+
+  char *ppp=strchr(shift_sele,xkey);
+  if (!(kbstate&LockMask) && ppp) {
+	int v = ppp - shift_sele;
+	KeySym k = noshi_sele[v];
+	if (!phkbm.phokbm[k][0].num)
+      return FALSE;
+  }
+
   int c = shift_key_idx(pho_selkey, xkey);
   if (c < 0) {
     close_selection_win();
@@ -1333,77 +1442,78 @@ void tsin_scan_pre_select(gboolean b_incr);
 
 static int cursor_backspace()
 {
-        close_selection_win();
-        poo.ityp3_pho=0;
-        tss.pre_selN = 0;
-        gboolean pho_cleared;
-        pho_cleared=FALSE;
-		int j;
+	dbg("cursor_backspace\n");
+	close_selection_win();
+	poo.ityp3_pho=0;
+	tss.pre_selN = 0;
+	gboolean pho_cleared;
+	pho_cleared=FALSE;
+	int j;
 
-        if (pin_juyin) {
-          for(j=sizeof(poo.inph)-1;j>=0;j--) {
-            if (poo.inph[j]) {
-              poo.inph[j]=0;
-              pho_cleared = TRUE;
-			  if (j==0)
-				clrin_pho();
-              break;
-            }
-          }
-        } else {
-          for(j=3;j>=0;j--)
-            if (poo.typ_pho[j]) {
-              poo.typ_pho[j]=0;
-              poo.inph[j]=0;
-              pho_cleared = TRUE;
-              break;
-            }
-        }
+	if (pin_juyin) {
+	  for(j=sizeof(poo.inph)-1;j>=0;j--) {
+		if (poo.inph[j]) {
+		  poo.inph[j]=0;
+		  pho_cleared = TRUE;
+		  if (j==0)
+			clrin_pho();
+		  break;
+		}
+	  }
+	} else {
+	  for(j=3;j>=0;j--)
+		if (poo.typ_pho[j]) {
+		  poo.typ_pho[j]=0;
+		  poo.inph[j]=0;
+		  pho_cleared = TRUE;
+		  break;
+		}
+	}
 
-        if (pho_cleared) {
-//          dbg("pho cleared %d %d %d\n",tss.c_len, gcin_pop_up_win, typ_pho_empty());
-		  if (typ_pho_empty())
-			  bzero(poo.inph, sizeof(poo.inph));
+	if (pho_cleared) {
+//   dbg("pho cleared %d %d %d\n",tss.c_len, gcin_pop_up_win, typ_pho_empty());
+	  if (typ_pho_empty())
+		  bzero(poo.inph, sizeof(poo.inph));
 
-          disp_in_area_pho_tsin();
-          tsin_scan_pre_select(TRUE);
+	  disp_in_area_pho_tsin();
+	  tsin_scan_pre_select(TRUE);
 
-          if (!tss.c_len && gcin_pop_up_win && typ_pho_empty())
-            hide_win0();
-          return 1;
-        }
+	  if (!tss.c_len && gcin_pop_up_win && typ_pho_empty())
+		hide_win0();
+	  return 1;
+	}
 
-        if (!tss.c_idx)
-          return 0;
+	if (!tss.c_idx)
+	  return 0;
 
-        clrcursor();
-        tss.c_idx--;
-//        pst=k=tss.chpho[tss.c_idx].psta;
+	clrcursor();
+	tss.c_idx--;
+//  pst=k=tss.chpho[tss.c_idx].psta;
 
-		int k;
-        for(k=tss.c_idx;k<tss.c_len;k++) {
-          tss.chpho[k]=tss.chpho[k+1];
-          if (tss.chpho[k+1].ch == tss.chpho[k+1].cha)
-            tss.chpho[k].ch = tss.chpho[k].cha;
-        }
+	int k;
+	for(k=tss.c_idx;k<tss.c_len;k++) {
+	  tss.chpho[k]=tss.chpho[k+1];
+	  if (tss.chpho[k+1].ch == tss.chpho[k+1].cha)
+		tss.chpho[k].ch = tss.chpho[k].cha;
+	}
 
-        tss.c_len--;
-        init_chpho_i(tss.c_len);
-		call_tsin_parse();
-		compact_win0();
+	tss.c_len--;
+	init_chpho_i(tss.c_len);
+	call_tsin_parse();
+	compact_win0();
 
-        if (!tss.c_idx) {
-          clear_match();
-        } else {
-          tsin_scan_pre_select(TRUE);
-        }
+	if (!tss.c_idx) {
+	  clear_match();
+	} else {
+	  tsin_scan_pre_select(TRUE);
+	}
 
-        disp_ph_sta();
+	disp_ph_sta();
 
-        if (!tss.c_len && gcin_pop_up_win)
-          hide_win0();
+	if (!tss.c_len && gcin_pop_up_win)
+	  hide_win0();
 
-        return 1;
+	return 1;
 }
 
 
@@ -1420,6 +1530,7 @@ void pho_play(phokey_t key);
 
 int tsin_pho_sel(int c)
 {
+  dbg("tsin_pho_sel\n");
   char *sel_text;
   int is_pho_phrase;
   int len = fetch_user_selection(c, &sel_text, &is_pho_phrase);
@@ -1444,12 +1555,17 @@ int tsin_pho_sel(int c)
     if (len == 1) {
       hide_selections_win();
       tss.ph_sta = -1;
+	  dbg("tss.c_len:%d %d\n",tss.c_len, gcin_buffer_select_char_auto_right);
+	  if (gcin_buffer_select_char_auto_right) {
+		   cursor_right();
+	  }
       return 0;
     }
     else
       tss.ph_sta=-1;
 
     hide_selections_win();
+
   }
 
   return 1;
@@ -1477,7 +1593,7 @@ gboolean tsin_page_down()
 
   tss.pho_menu_idx = 0;
   tss.current_page = tss.current_page + phkbm.selkeyN;
-  if (tss.current_page >= phrase_count + pho_count)
+  if (tss.current_page >= tss.phrase_count + tss.pho_count)
     tss.current_page = 0;
 
   disp_current_sel_page();
@@ -1498,8 +1614,8 @@ void open_select_pho()
 
   int idx = tss.c_idx==tss.c_len?tss.c_len-1:tss.c_idx;
 
-  tss.pho_menu_idx = phrase_count;
-  if (phrase_count < phkbm.selkeyN-1) {
+  tss.pho_menu_idx = tss.phrase_count;
+  if (tss.phrase_count < phkbm.selkeyN-1) {
 	char *p = pho_idx_str(tss.startf);
 	if (utf8_eq(p, tss.chpho[idx].ch))
       tss.pho_menu_idx++;
@@ -1518,19 +1634,26 @@ static void tsin_create_win_save_phrase(int idx0, int len)
   WSP_S wsp[MAX_PHRASE_LEN];
   int i;
   for(i=0;i<len;i++) {
-    memcpy(wsp[i].ch, tss.chpho[i+idx0].ch, CH_SZ);
+	char *ch = tss.chpho[i+idx0].ch;
+
+    memcpy(wsp[i].ch, ch, CH_SZ);
     wsp[i].key = tss.chpho[i+idx0].pho;
   }
   create_win_save_phrase(wsp, len);
 }
 
-void tsin_sel_cursor_LR(gboolean is_inc)
+int tsin_sel_max_N()
 {
-  is_inc ^= pho_candicate_R2L;
   int N;
-  N = phrase_count + pho_count - tss.current_page;
+  N = tss.phrase_count + tss.pho_count - tss.current_page;
   if (N > phkbm.selkeyN)
     N = phkbm.selkeyN;
+  return N;
+}
+
+void tsin_sel_cursor_inc_dec(gboolean is_inc)
+{
+  int N = tsin_sel_max_N();
 
   if (is_inc) {
     tss.pho_menu_idx = (tss.pho_menu_idx+1) % N;
@@ -1543,7 +1666,31 @@ void tsin_sel_cursor_LR(gboolean is_inc)
   disp_current_sel_page();
 }
 
-int feedkey_pp(KeySym xkey, int kbstate)
+int is_pho_key(KeySym xkey)
+{
+  return xkey < 127 && phkbm.phokbm[xkey][0].num;
+}
+
+extern int win1_rowN;
+
+static void pho_menu_idx_left()
+{
+  int N = tsin_sel_max_N();
+  tss.pho_menu_idx = (tss.pho_menu_idx - win1_rowN + N) % N;
+  disp_current_sel_page();
+}
+
+static void pho_menu_idx_right()
+{
+  int N = tsin_sel_max_N();
+  tss.pho_menu_idx = (tss.pho_menu_idx + win1_rowN) % N;
+  disp_current_sel_page();
+}
+
+void tsin_en_scan_pre_select();
+extern int capslock_on;
+
+int feedkey_tsin(KeySym xkey, int kbstate)
 {
   char ctyp=0;
   static u_int ii;
@@ -1554,18 +1701,14 @@ int feedkey_pp(KeySym xkey, int kbstate)
   char kno;
   int caps_eng_tog = tsin_chinese_english_toggle_key == TSIN_CHINESE_ENGLISH_TOGGLE_KEY_CapsLock;
   int status=0;
-
+  gboolean key_has_pho = xkey < 127 && (phkbm.phokbm[xkey][0].num || phkbm.phokbm[xkey][0].typ);
 
 //  dbg("feedkey_pp %x %x\n", xkey, kbstate);
 //  if (xkey=='1')
 //    dbg("aaa\n");
 
   if (caps_eng_tog) {
-    gboolean new_tsin_pho_mode =!(kbstate&LockMask);
-#if UNIX
-    if (xkey == XK_Caps_Lock)
-      new_tsin_pho_mode = !new_tsin_pho_mode; // X11 caplock is not on/off immediately
-#endif
+    gboolean new_tsin_pho_mode = !capslock_on;
     if (current_CS->tsin_pho_mode != new_tsin_pho_mode) {
       close_selection_win();
       tsin_set_eng_ch(new_tsin_pho_mode);
@@ -1575,7 +1718,6 @@ int feedkey_pp(KeySym xkey, int kbstate)
 //  key_press_time = 0;
 
    if (kbstate & (Mod1Mask|Mod4Mask|Mod5Mask)) {
-//     dbg("ret\n");
      return 0;
    }
 
@@ -1596,23 +1738,28 @@ int feedkey_pp(KeySym xkey, int kbstate)
      key_press_time_ctrl = 0;
    }
 
+
    if (!tsin_pho_mode() && !tss.c_len && gcin_pop_up_win && xkey!=XK_Caps_Lock) {
-     hide_win0();
+//     hide_win0();
      gboolean is_ascii = (xkey>=' ' && xkey<0x7f) && !ctrl_m;
 
      if (caps_eng_tog && is_ascii) {
        if (gcin_capslock_lower)
          case_inverse(&xkey, shift_m);
-       send_ascii(xkey);
-       return 1;
+//       send_ascii(xkey);
+//       return 1;
      }
-     else {
+//     else
+     {
        if (tss.tsin_half_full && is_ascii) {
          send_text(half_char_to_full_char(xkey));
          return 1;
        }
        else {
-         return 0;
+		 if (!en_pre_select)
+           return 0;
+         if (!(xkey>='A' && xkey<='Z' || xkey>='a' && xkey<='z'))
+           return 0;
        }
      }
    }
@@ -1622,10 +1769,17 @@ int feedkey_pp(KeySym xkey, int kbstate)
 
    switch (xkey) {
      case XK_Escape:
+	   if (tss.sel_pho) {
+		  close_selection_win();
+		  return TRUE;
+	   }
+
        tsin_reset_in_pho0();
        if (typ_pho_empty()) {
          if (!tss.c_len)
            return 0;
+         if (gcin_escape_clear_edit_buffer)
+           goto clear_edit_buffer;
          if (!o_sel_pho && tsin_tab_phrase_end) {
            goto tab_phrase_end;
          }
@@ -1635,10 +1789,9 @@ int feedkey_pp(KeySym xkey, int kbstate)
      case XK_Return:
      case XK_KP_Enter:
         if (shift_m) {
-          if (!tss.c_len)
+          if (!tss.c_len || tss.c_len == tss.c_idx)
             return 0;
-		  int start = tss.c_len == tss.c_idx?0:tss.c_idx;
-          tsin_create_win_save_phrase(start,  tss.c_len - start);
+          tsin_create_win_save_phrase(tss.c_idx,  tss.c_len - tss.c_idx);
           move_cursor_end();
           return 1;
         } else {
@@ -1657,9 +1810,16 @@ int feedkey_pp(KeySym xkey, int kbstate)
 #if UNIX
      case XK_KP_Home:
 #endif
+        if (tss.sel_pho) {
+          tss.pho_menu_idx=0;
+          disp_current_sel_page();
+          return TRUE;
+        }
+
         close_selection_win();
         if (!tss.c_len)
           return 0;
+
         clrcursor();
         tss.c_idx=0;
         drawcursor();
@@ -1668,6 +1828,12 @@ int feedkey_pp(KeySym xkey, int kbstate)
 #if UNIX
      case XK_KP_End:
 #endif
+        if (tss.sel_pho) {
+          tss.pho_menu_idx=tsin_sel_max_N()-1;
+          disp_current_sel_page();
+          return TRUE;
+        }
+
         close_selection_win();
         if (!tss.c_len)
           return 0;
@@ -1678,7 +1844,7 @@ int feedkey_pp(KeySym xkey, int kbstate)
      case XK_KP_Left:
 #endif
         if (tss.sel_pho) {
-          tsin_sel_cursor_LR(FALSE);
+		  pho_menu_idx_left();
           return TRUE;
         } else
           return cursor_left();
@@ -1687,7 +1853,7 @@ int feedkey_pp(KeySym xkey, int kbstate)
      case XK_KP_Right:
 #endif
         if (tss.sel_pho) {
-          tsin_sel_cursor_LR(TRUE);
+		  pho_menu_idx_right();
           return TRUE;
         } else
           return cursor_right();
@@ -1753,12 +1919,7 @@ tab_phrase_end:
          return tss.c_len>0;
        }
 
-       int N;
-       N = phrase_count + pho_count - tss.current_page;
-       if (N > phkbm.selkeyN)
-         N = phkbm.selkeyN;
-       if (tss.pho_menu_idx >=pho_candicate_col_N)
-         tss.pho_menu_idx-=pho_candicate_col_N;
+	   tsin_sel_cursor_inc_dec(FALSE);
        disp_current_sel_page();
        return 1;
      case XK_Prior:
@@ -1777,18 +1938,25 @@ tab_phrase_end:
          return win_sym_page_up();
        }
      case XK_space:
-       if (!tss.c_len && !poo.ityp3_pho && !poo.typ_pho[0] && !poo.typ_pho[1] && !poo.typ_pho[2]
-           && tss.tsin_half_full) {
-         send_text(_(_L("　")));
+       if (!tss.c_len && !poo.ityp3_pho && typ_pho_empty3() /* && tss.tsin_half_full */ ) {
+send_space:
+         send_text(tss.tsin_half_full?_(_L("　")):" ");
          return 1;
        }
 
-       if (tsin_space_opt == TSIN_SPACE_OPT_INPUT && !poo.typ_pho[0] && !poo.typ_pho[1] && !poo.typ_pho[2] && !poo.ityp3_pho && !tss.sel_pho) {
-         if (tss.c_len)
+       if (tsin_pho_mode() && tss.c_len && (tsin_space_opt & (TSIN_SPACE_OPT_INPUT|TSIN_SPACE_OPT_FLUSH_EDIT)) && typ_pho_empty3() && !poo.ityp3_pho && !tss.sel_pho) {
+//         if (tss.c_len)
            flush_tsin_buffer();
 
          close_selection_win();
+
+         if (tsin_space_opt == TSIN_SPACE_OPT_FLUSH_EDIT)
+           return TRUE;
+#if 0
          goto asc_char;
+#else
+		 goto send_space;
+#endif
        }
 
        if (!tsin_pho_mode())
@@ -1820,11 +1988,7 @@ change_char:
          if (xkey == XK_space)
            tsin_page_down();
          else {
-           int N = phrase_count + pho_count - tss.current_page;
-           if (N > phkbm.selkeyN)
-             N = phkbm.selkeyN;
-           if (tss.pho_menu_idx+pho_candicate_col_N < N)
-             tss.pho_menu_idx = tss.pho_menu_idx+pho_candicate_col_N;
+           tsin_sel_cursor_inc_dec(TRUE);
            disp_current_sel_page();
          }
        }
@@ -1845,20 +2009,34 @@ change_char:
 		 return win_sym_page_down();
 	   }
      case '\'':  // single quote
-       if (phkbm.phokbm[xkey][0].num && !pin_juyin)
+       if (phkbm.phokbm[xkey][0].num && !pin_juyin || !tsin_pho_mode())
          goto other_keys;
        else {
-	 return pre_punctuation_hsu(xkey);
+	     return pre_punctuation_hsu(xkey);
        }
+#if 0
+     case '.':
+     case '?':
+     case ',':
+     case '!':
+     case ':':
+     case ';':
+	   if (tsin_pho_mode() || ctrl_m || tss.c_idx != tss.c_len)
+         goto other_keys;
+       flush_tsin_buffer();
+       send_ascii(xkey);
+       return TRUE;
+#endif
      case 'q':
      case 'Q':
        if (b_hsu_kbm && tsin_pho_mode())
          goto change_char;
      default:
 other_keys:
-       if ((kbstate & ControlMask)) {
+       if (ctrl_m) {
          if (xkey=='u') {
            if (tss.c_len) {
+clear_edit_buffer:
              clear_tsin_buffer();
              if (gcin_pop_up_win)
                hide_win0();
@@ -1921,11 +2099,11 @@ other_keys:
          xkey_lcase = xkey - XK_KP_0 + '0';
 
        gboolean use_pre_sel;
-       use_pre_sel = tss.pre_selN && !tss.sel_pho && xkey < 127 && !phkbm.phokbm[xkey][0].num;
+       use_pre_sel = tss.pre_selN && !tss.sel_pho && xkey < 127 && (!phkbm.phokbm[xkey][0].num || !tsin_pho_mode());
 
        char *pp;
-       if ((pp=strchr(pho_selkey,xkey_lcase)) && (tss.sel_pho || tss.ctrl_pre_sel || use_pre_sel)) {
-         int c=pp-pho_selkey;
+       if ((pp=strchr(wselkey, xkey_lcase)) && (tss.sel_pho || tss.ctrl_pre_sel || use_pre_sel)) {
+         int c=pp-wselkey;
 
          if (tss.sel_pho) {
            if (tsin_pho_sel(c))
@@ -1949,6 +2127,8 @@ other_keys:
    KeySym key_pad;
    key_pad = keypad_proc(xkey);
 
+//   dbg("key_pad %d\n", key_pad);
+
    if (!xkey || (xkey > 0x7e && !key_pad))
      return 0;
 
@@ -1956,8 +2136,8 @@ other_keys:
      return 0;
 
    if (!tsin_pho_mode() || poo.typ_pho[0]!=BACK_QUOTE_NO && (shift_m || key_pad ||
-       (!phkbm.phokbm[xkey][0].num && !phkbm.phokbm[xkey][0].typ))) {
-       if (tsin_pho_mode() && !shift_m && strchr(hsu_punc, xkey) && !phkbm.phokbm[xkey][0].num) {
+       (!key_has_pho))) {
+       if (tsin_pho_mode() && !shift_m && strchr(hsu_punc, xkey) && !is_pho_key(xkey)) {
          if (pre_punctuation_hsu(xkey))
            return 1;
        }
@@ -1965,22 +2145,32 @@ other_keys:
        if (key_pad)
          xkey = key_pad;
 asc_char:
-        if (shift_m) {
-          if (pre_sel_handler(xkey)) {
+        if (shift_m && tsin_pho_mode()) {
+          if (pre_sel_handler(xkey, kbstate)) {
             call_tsin_parse();
             return 1;
           }
 
-          if (tsin_pho_mode() && pre_punctuation(xkey))
+          if (tsin_shift_punc /* && tsin_pho_mode() */ && pre_punctuation(xkey))
             return 1;
         }
 
-        if (shift_m && tsin_pho_mode())  {
-          char *ppp=strchr(ochars,xkey);
+		if (!key_has_pho && tsin_pho_mode() && !tss.c_len) {
+			  return FALSE;
+		}
 
-          if (!(kbstate&LockMask) && ppp && !((ppp-ochars) & 1))
-            xkey=*(ppp+1);
+        if (xkey > 127)
+          return 0;
 
+		if (shift_m && tsin_pho_mode())  {
+#if 0
+          if (key_has_pho) {
+            char *ppp=strchr(ochars,xkey);
+            if (!(kbstate&LockMask) && ppp && !((ppp-ochars) & 1)) {
+				 xkey = *(ppp+1);
+			}
+	      }
+#endif
         } else {
           if (!tsin_pho_mode() && tsin_chinese_english_toggle_key == TSIN_CHINESE_ENGLISH_TOGGLE_KEY_CapsLock
               && gcin_capslock_lower) {
@@ -1988,8 +2178,6 @@ asc_char:
           }
         }
 
-        if (xkey > 127)
-          return 0;
         char tstr[CH_SZ + 1];
         bzero(tstr, sizeof(tstr));
 
@@ -2001,26 +2189,43 @@ asc_char:
           tstr[0] = tt;
         }
 
-        if (!tss.c_len) {
+        dbg("en %s\n", tstr);
+
+        if (!tss.c_len && !en_pre_select) {
+          dbg("ret a\n");
           send_text(tstr);
           return 1;
         }
 
+		if (tss.c_idx == tss.c_len && (!tsin_pho_mode() || !key_has_pho) && strchr("!@#$%^&*()-_+{}\\|;:\"<>?,./[]=", xkey)) {
+			flush_tsin_buffer();
+			send_text(tstr);
+			return 1;
+		}
+
         shift_ins();
 
         memcpy(tss.chpho[tss.c_idx].ch, tstr, CH_SZ);
 
         set_fixed(tss.c_idx, 1);
-        phokey_t tphokeys[32];
-        tphokeys[0]=0;
-        utf8_pho_keys(tss.chpho[tss.c_idx].ch, tphokeys);
+//        phokey_t tphokeys[32];
+//        tphokeys[0]=0;
+//        utf8_pho_keys(tss.chpho[tss.c_idx].ch, tphokeys);
 
         disp_char_chbuf(tss.c_idx);
-        tss.chpho[tss.c_idx].pho=tphokeys[0];
+		phokey_t pk = utf8_pho_key(tstr);
+		if (!pk)
+			pk = 1; //.
+        tss.chpho[tss.c_idx].pho= pk;
+
         tss.c_idx++;
         if (tss.c_idx < tss.c_len)
           prbuf();
 
+        if (en_pre_select) {
+          tsin_en_scan_pre_select();
+		}
+
         if (gcin_pop_up_win)
           show_win0();
 
@@ -2056,7 +2261,7 @@ llll2:
        if (!tsin_tone_char_input && !poo.typ_pho[0] && !poo.typ_pho[1] && !poo.typ_pho[2]) {
          clrin_pho_tsin();
          dbg("no pho input\n");
-         return TRUE;
+         return tss.c_len>0;
        }
      }
 
@@ -2205,14 +2410,19 @@ void tsin_remove_last()
 #endif
 
 
-gboolean save_phrase_to_db2(CHPHO *chph, int len)
+gboolean save_phrase_to_db2(CHPHO *chph, int len, gboolean en_phrase)
 {
    phokey_t pho[MAX_PHRASE_LEN];
    char ch[MAX_PHRASE_LEN * CH_SZ * 2];
-
    chpho_extract_cha(chph, len, pho, ch);
 
-   return save_phrase_to_db(pho, ch, len, 1);
+   dbg("save_phrase_to_db2 en_phrase:%d %s\n", en_phrase, ch);
+
+   if (en_phrase) {
+     return save_phrase_to_db(&en_hand, ch, NULL, len, 1);
+   } else {
+     return save_phrase_to_db(&tsin_hand, pho, ch, len, 1);
+   }
 }
 
 #include "im-client/gcin-im-client-attr.h"
@@ -2281,7 +2491,7 @@ fin:
 
 int tsin_reset()
 {
-//  dbg("tsin_reset\n");
+  dbg("tsin_reset\n");
 #if UNIX
   if (!gwin0)
     return 0;
@@ -2290,6 +2500,9 @@ int tsin_reset()
   tsin_reset_in_pho0();
   clear_tsin_buffer();
 
+  if (gcin_pop_up_win)
+	hide_win0();
+
   return v;
 }
 
